import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;
import com.aspose.cells.Workbook;
import com.aspose.cells.*;

class ConsoleDemo
{
   private static final String DB_URL = 
      "jdbc:odbc:driver={Microsoft Access Driver (*.mdb)};DBQ=./Database/Northwind.mdb";
   private static final String TEMPLATE_FILE_PATH = "./Designer/Northwind.xls";
   private static final String OUTPUT_FILE_PATH = "./Output/";
   
   public static void main(String[] args) throws Exception
   {
      Class.forName("sun.jdbc.odbc.JdbcOdbcDriver"); 
      String outputPath = OUTPUT_FILE_PATH;
      // Any of the following lines can run separately.
      int type = 0;
      if(args != null && args.length != 0){
         try
         {
            type = Integer.parseInt(args[0]);
         }catch(NumberFormatException nfe){
            nfe.printStackTrace();
         }
      }
      create(type,outputPath);     
   }
   
   private static void create(int type,String outputPath) throws SQLException, IOException
   {

    switch (type)
    {
    case 1:
       createProductsList(outputPath);
       break;
    case 2:
       createCatalog(outputPath);
       break;
    case 3:
       createCatalogSubreport(outputPath);
       break;
    case 4:
       createCustomerLabels(outputPath);
       break;
    case 5:
       createEmployeeSales(outputPath);
       break;
    case 6:
       createInvoice(outputPath);
       break;
    case 7:
       createProductsByCategory(outputPath);
       break;
    case 8:
       createSalesByCategorySubreport(outputPath);
       break;
    case 9:
       createSalesByYear(outputPath);
       break;
    case 10:
       createSalesByYearSubreport(outputPath);
       break;
    case 11:
       createSalesTotals(outputPath);
       break;
    case 12:
       CreateSummaryByQuarter(outputPath); 
       break;
    default :
       createProductsList(outputPath);
       createCatalog(outputPath);
       createCatalogSubreport(outputPath);
       createCustomerLabels(outputPath);
       createEmployeeSales(outputPath);
       createInvoice(outputPath);
       createProductsByCategory(outputPath);
       createSalesByCategorySubreport(outputPath);
       createSalesByYear(outputPath);
       createSalesByYearSubreport(outputPath);
       createSalesTotals(outputPath);
       CreateSummaryByQuarter(outputPath); 
    }

   }
   ////////////////////////// BEGIN OF Alphabetical List of Products  ///////////////////////
   private static void createProductsList(String outputPath) throws SQLException, IOException
   {
      Workbook book = new Workbook();
      book.read(TEMPLATE_FILE_PATH);
      for(int i = book.getNumerOfSheets() - 1; i > 0; --i) {
         book.removeSheet(i);
      }
      Worksheet sheet =book.getSheet(0);
      sheet.setName("Product List");      
      
      // read from database
      Connection conn = null;
      ResultSet rs = null;
      Statement stmt = null;
      try {
         conn = DriverManager.getConnection(DB_URL);
         stmt = conn.createStatement();
         rs = stmt.executeQuery(
               "SELECT DISTINCTROW Products.ProductName, Categories.CategoryName, " + 
               " Products.QuantityPerUnit, Products.UnitsInStock " +
               " FROM Categories INNER JOIN Products " + 
               " ON Categories.CategoryID = Products.CategoryID " + 
               " WHERE (((Products.Discontinued) = No)) " +
               " Order by Products.ProductName");
         
         int rowIndex = 5;
         while(rs.next()) {
            sheet.getCell(rowIndex, 1).setValue(rs.getString(1));
            sheet.getCell(rowIndex, 2).setValue(rs.getString(2));
            sheet.getCell(rowIndex, 3).setValue(rs.getString(3));
            sheet.getCell(rowIndex, 4).setValue(rs.getInt(4));
            ++rowIndex;
         }
      }
      finally {
         if(rs != null)
            rs.close();
         if(stmt != null)
            stmt.close();
         if(conn != null)
            conn.close();
      }
     
      book.write(outputPath+"AlphabeticalList.xls");
   }
   //////////////////////////// END OF Alphabetical List of Products  //////////////////////
   
   
   //////////////////////////// BEGIN OF CATALOG ///////////////////////////////////////////
   private static void createCatalog(String outputPath) throws SQLException, IOException
   {
      Workbook book = new Workbook();
      book.read(TEMPLATE_FILE_PATH);
      Worksheet sheet = book.getSheet("Sheet2");
      sheet.setName("Catalog");
      
      // remove unuseful sheets
      for(int i = book.getNumerOfSheets() - 1; i >= 0 ; --i) {
         if(book.getSheet(i) != sheet)
            book.removeSheet(i);
      }
      
      // customize colors, fonts, and styles
      Style styleOfCategoryName, styleOfDescription, styleOfProductName, styleOfTitle, styleOfNumber;
      Color lightGray = new Color(211, 211, 211);
      
      book.getPalette().setColor(55, lightGray); // light gray
      Font font;
      
      font = new Font();
      font.setName("Times New Roman");
      font.setColor(Color.BLUE);
      font.setSize(14);
      font.setBold(true);
      styleOfCategoryName = book.createStyle();
      styleOfCategoryName.setFont(font);
      
      font = new Font();
      font.setName("Times New Roman");
      font.setColor(Color.BLUE);
      font.setItalic(true);
      styleOfDescription = book.createStyle();
      styleOfDescription.setFont(font);
      
      font = new Font();
      font.setBold(true);
      styleOfProductName = book.createStyle();
      styleOfProductName.setFont(font);
      
      font = new Font();
      font.setBold(true);
      font.setItalic(true);
      styleOfTitle = book.createStyle();
      styleOfTitle.setFont(font);
      styleOfTitle.setPatternStyle(PatternType.SOLID);
      styleOfTitle.setPatternColor(lightGray);

      font = new Font();
      font.setName("Times New Roman");
      styleOfNumber = book.createStyle();
      styleOfNumber.setNumber(8);
      
      Connection conn = null;
      Statement stmt = null;
      PreparedStatement pstmt = null;
      ResultSet rs_cata = null;
      ResultSet rs_prod = null;
      
      try {
         conn = DriverManager.getConnection(DB_URL);
         stmt = conn.createStatement();
         rs_cata = stmt.executeQuery(
               "SELECT CategoryID, CategoryName, Description FROM Categories");
         pstmt = conn.prepareStatement(
               "SELECT ProductName, ProductID, QuantityPerUnit, UnitPrice " + 
               " FROM Products" + 
               " WHERE categoryid = ?");
         
         int rowIndex = 55;
         while(rs_cata.next()) {
            rowIndex += 2;
            Row row;
            Cell cell;
            
            // category name
            row = sheet.getRow(rowIndex);
            row.setHeight(20);
            cell = row.getCell(1);
            cell.setStyle(styleOfCategoryName);
            cell.setValue(rs_cata.getString(2));
            
            // description
            row = sheet.getRow(++rowIndex);
            cell = row.getCell(1);
            cell.setStyle(styleOfDescription);
            cell.setValue(rs_cata.getString(3));
            
            // query product
            pstmt.setInt(1, rs_cata.getInt(1));
            rs_prod = pstmt.executeQuery();
            
            rowIndex += 2;
            row = sheet.getRow(rowIndex);
            row.setStyle(1, 4, styleOfTitle);
            
            int rowNum = sheet.importResultSet(rs_prod, rowIndex, 1, -1, -1, true);
            sheet.setStyle(rowIndex + 1, rowIndex + 1 + rowNum - 1, 1, 1, styleOfProductName);
            sheet.setStyle(rowIndex + 1, rowIndex + 1 + rowNum - 1, 4, 4, styleOfNumber);
            
            rowIndex += rowNum;
            rs_prod.close();
            rs_prod = null;
         }
      }
      finally {
         if(rs_cata != null)
            rs_cata.close();
         if(rs_prod != null)
            rs_prod.close();
         if(pstmt != null)
            pstmt.close();
         if(stmt != null)
            stmt.close();
         if(conn != null)
            conn.close();
      }
      book.write(outputPath+"Catalog.xls");
   }
   /////////////////////////// END OF CATALOG //////////////////////////////////////////
   
   
   ////////////////////////// BEGIN OF Catalog Subreport ///////////////////////////////
   private static void createCatalogSubreport(String outputPath) throws IOException, SQLException
   {
      Workbook book = new Workbook();
      book.read(TEMPLATE_FILE_PATH);
      Worksheet sheet = book.getSheet("Sheet3");
      sheet.setName("Catalog Subreport");
      
      // remove unuseful sheets
      for(int i = book.getNumerOfSheets() - 1; i >= 0 ; --i) {
         if(book.getSheet(i) != sheet)
            book.removeSheet(i);
      }
      
      Connection conn = null;
      Statement stmt = null;
      ResultSet rs = null;
      
      try {
         conn = DriverManager.getConnection(DB_URL);
         stmt = conn.createStatement();
         rs = stmt.executeQuery(
               "SELECT ProductName, ProductID, QuantityPerUnit, UnitPrice" + 
               " FROM Products" + 
               " ORDER BY ProductName");
         sheet.importResultSet(rs, 0, 1, -1, -1, false);
      }
      finally {
         if(rs != null)
            rs.close();
         if(stmt != null)
            stmt.close();
         if(conn != null)
            conn.close();         
      }
      book.write(outputPath+"CatalogSubreport.xls");
   }
   //////////////////////////// END OF Catalog Subreport ///////////////////////////////
   
   
   //////////////////////////// BEGIN OF CUSTOMER LABELS ///////////////////////////////
   private static void createCustomerLabels(String outputPath) throws IOException, SQLException
   {
      Workbook book = new Workbook();
      book.read(TEMPLATE_FILE_PATH);
      Worksheet sheet = book.getSheet("Sheet4");
      sheet.setName("Customer Labels");
      
      // remove unuseful sheets
      for(int i = book.getNumerOfSheets() - 1; i >= 0 ; --i) {
         if(book.getSheet(i) != sheet)
            book.removeSheet(i);
      }
      
      Connection conn = null;
      Statement stmt = null;
      ResultSet rs = null;
      
      try {
         conn = DriverManager.getConnection(DB_URL);
         stmt = conn.createStatement();
         rs = stmt.executeQuery(
               "SELECT CompanyName, Address, City, Region, PostalCode, Country" + 
               " FROM Customers" + "" +
               " ORDER BY Country, Region");
         
         int sign = 0, rowIndex = 0;
         while(rs.next()) {
            int columnIndex = sign * 3;
            
            sheet.getCell(rowIndex, columnIndex).setValue(rs.getString(1));
            sheet.getCell(rowIndex + 1, columnIndex).setValue(rs.getString(2));
            
            StringBuilder builder = new StringBuilder();
            String tmp_string;
            builder.append(rs.getString(3));
            builder.append(rs.getString(4));
            builder.append(rs.getString(5));
            sheet.getCell(rowIndex + 2, columnIndex).setValue(builder.toString());
            
            sheet.getCell(rowIndex + 3, columnIndex).setValue(rs.getString(6));
            
            if(++sign == 3) {
               sign = 0;
               rowIndex += 5;
            }
         }
      }
      finally {
         if(rs != null)
            rs.close();
         if(stmt != null)
            stmt.close();
         if(conn != null)
            conn.close();         
      }
      book.write(outputPath+"CatalogSubreport.xls");
   }
   ///////////////////////////// END OF CUSTOMER LABELS ////////////////////////////////
   
   
   //////////////////////////// BEGIN OF Employee Sales by Country /////////////////////
   private static void createEmployeeSales(String outputPath) throws IOException, SQLException
   {
      Workbook book = new Workbook();
      book.read(TEMPLATE_FILE_PATH);
      
      Worksheet sheet_uk = book.getSheet("Sheet5");
      sheet_uk.setName("Employee Sales UK");
      
      Worksheet sheet_us = book.getSheet("Sheet6");
      sheet_us.setName("Employee Sales USA");      
      
      // remove unuseful sheets
      for(int i = book.getNumerOfSheets() - 1; i >= 0 ; --i) {
         Worksheet tmp = book.getSheet(i);
         if( (tmp != sheet_uk) && (tmp != sheet_us) )
            book.removeSheet(i);
      }
      
      Connection conn = null;
      Statement stmt = null;
      PreparedStatement pstmt = null;
      ResultSet rs_employee = null;
      ResultSet rs_order = null;
      
      ArrayList params_uk = new ArrayList();
      ArrayList params_us = new ArrayList();      
      try {
         conn = DriverManager.getConnection(DB_URL);
         stmt = conn.createStatement();
         rs_employee = stmt.executeQuery(
               "SELECT Country, EmployeeID, FirstName, LastName FROM Employees " +
               " ORDER BY Country, LastName, FirstName");
         pstmt = conn.prepareStatement(
               "SELECT Orders.OrderID, [Order Subtotals].Subtotal as SaleAmount" +
               " FROM Employees INNER " +
               " JOIN (Orders INNER JOIN [Order Subtotals] ON Orders.OrderID = " +
                  " [Order Subtotals].OrderID) " +
               " ON Employees.EmployeeID = Orders.EmployeeID" +
               " where Employees.EmployeeID = ?");
         
         int ukRowIndex = 6, usRowIndex = 6;
         Style style;
         
         // create a style for the header
         style = book.createStyle();
         style.setName("HeaderStyle");
         style.setBorderLine(BorderType.BOTTOM, BorderLineType.DOUBLE);
         style.setBorderLine(BorderType.TOP, BorderLineType.DOUBLE);
         style.getFont().setSize(12);
         style.getFont().setBold(true);
         style.setTextWrapped(true);
         style.setHAlignment(HorizontalAlignmentType.CENTRED);
         
         while(rs_employee.next()) {
            String country = rs_employee.getString(1);
            int employee_id = rs_employee.getInt(2);
            String employee_name = rs_employee.getString(4) + ", " + rs_employee.getString(3);
            
            Worksheet sheet;
            ArrayList params_xx;
            Cell cell;
            Row row;
            int rowIndex;
            if(country.equals("UK")) {
               sheet = sheet_uk;
               rowIndex = ukRowIndex;
               params_xx = params_uk;
            }
            else if(country.equals("USA")) {
               sheet = sheet_us;
               rowIndex = usRowIndex;
               params_xx = params_us;
            }
            else
               continue;
            
            cell = sheet.getCell(rowIndex - 2, 0);
            cell.setValue("SalePerson: " + employee_name);
            style = book.createStyle();
            style.getFont().setBold(true);
            style.getFont().setSize(12);
            cell.setStyle(style);
            
            sheet.getRow(rowIndex - 2).setHeight(19);
            sheet.getRow(rowIndex - 1).setHeight(4);
            sheet.getRow(rowIndex).setHeight(48);
            
            style = book.getNamedStyle("HeaderStyle");
            row = sheet.getRow(rowIndex);
            row.setStyle(1, 4, style);
            row.getCell(1).setValue("Order ID:");
            row.getCell(2).setValue("Sales Amount:");
            row.getCell(3).setValue("Percent of Salesperson's Total:");
            row.getCell(4).setValue("Percent of Country Total:");
            ++rowIndex;
            
            pstmt.setInt(1, employee_id);
            rs_order = pstmt.executeQuery();
            int rowNum = sheet.importResultSet(rs_order, rowIndex, 1, false);
            
            // stores the start row index and the row count
            int[] rowAndRowNum = new int[2];
            rowAndRowNum[0] = rowIndex;
            rowAndRowNum[1] = rowNum;
            params_xx.add(rowAndRowNum);
            
            rowIndex += rowNum;
            rowIndex += 4;
            rs_order.close();
            rs_order = null;
            
            if(country.equals("UK")) {
               ukRowIndex = rowIndex;
            }
            else if(country.equals("USA")) {
               usRowIndex = rowIndex;
            }            
         }
      }
      finally {
         if(rs_employee != null)
            rs_employee.close();
         if(rs_order != null)
            rs_order.close();
         if(pstmt != null)
            pstmt.close();
         if(stmt != null)
            stmt.close();
         if(conn != null)
            conn.close();         
      }
      
      // Prepare the style for the "Exceeded Goal"
      Style styleOfExceededGoal = book.createStyle();
      Font font = styleOfExceededGoal.getFont();
      font.setBold(true);
      font.setItalic(true);
      font.setSize(12);
      font.setColor(Color.RED);
      
      // Calculates sales percentages
      for(int i = 0; i < book.getNumerOfSheets(); ++i) {
         Worksheet sheet = book.getSheet(i);
         ArrayList params = sheet.getName().endsWith("UK") ? params_uk : params_us;
         
         double country_total = 0;
         double person_total = 0;
         for(int j = 0; j < params.size(); ++j) {
            int[] rowIndexAndCount = (int[])params.get(j);
            int startRowIndex = rowIndexAndCount[0];
            int rowCount = rowIndexAndCount[1];
            
            person_total = 0;
            for(int m = startRowIndex; m < startRowIndex + rowCount; ++m) {
               person_total += ((Double)(sheet.getCell(m, 2).getValue())).doubleValue();
            }
            if(person_total > 5000) {
               Cell cell = sheet.getCell(startRowIndex - 3, 3);
               cell.setValue("Exceeded Goal!");
               cell.setStyle(styleOfExceededGoal);
            }
            sheet.getCell(startRowIndex + rowCount, 2).setValue(person_total);
            country_total += person_total;
         }
         
         for(int j = 0; j < params.size(); ++j) {
            int[] rowIndexAndCount = (int[])params.get(j);
            int startRowIndex = rowIndexAndCount[0];
            int rowCount = rowIndexAndCount[1];
            
            person_total = ((Double)(sheet.getCell(startRowIndex + rowCount, 2)
                  .getValue())).doubleValue();
            for(int m = startRowIndex; m < startRowIndex + rowCount; ++m) {
               // person percentage
               double single_sale = ((Double)(sheet.getCell(m, 2)
                     .getValue())).doubleValue();
               sheet.getCell(m, 3).setValue(single_sale / person_total);
               sheet.getCell(m, 4).setValue(single_sale / country_total);
            }
            sheet.getCell(startRowIndex + rowCount, 4).setValue(person_total / country_total);
         }
         
      }
      
      book.write(outputPath+"EmployeeSales.xls");      
   }
   ///////////////////////////// END OF Employee Sales by Country //////////////////////
   
   
   //////////////////////////// BEGIN OF Create Invoice /////////////////////////////////
   private static void createInvoice(String outputPath) throws IOException, SQLException
   {
      Workbook book = new Workbook();
      Worksheet sheet = book.getSheet(0);
      sheet.setName("Invoice");
      setColumnWidth(sheet);
      createInvoiceStyles(book);
            
      Connection conn = null;
      Statement stmt = null;
      PreparedStatement pstmt = null;
      ResultSet rs_order = null;
      ResultSet rs_invoice = null;
      
      try {
         conn = DriverManager.getConnection(DB_URL);
         stmt = conn.createStatement();
         rs_order = stmt.executeQuery(
               "SELECT OrderID FROM Orders ORDER BY OrderID DESC");
         pstmt = conn.prepareStatement(
               "SELECT DISTINCTROW Invoices.* FROM Invoices WHERE Invoices.OrderID = ?",
               ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
         
         int rowIndex = 0;
         while(rs_order.next()) {
            // query invoices
            int orderID = rs_order.getInt(1);
            pstmt.setInt(1, orderID);
            rs_invoice = pstmt.executeQuery();
            
            if(!rs_invoice.next())
               continue;
                        
            createInvoiceHeader(book, sheet, rs_invoice, rowIndex);
            rowIndex += 11;
            createOrder(book, sheet, rowIndex, orderID, rs_invoice);
            rowIndex += 4;
            rowIndex += createOrderDetail(book, sheet, rowIndex, rs_invoice) + 1;
            
            
            rs_invoice.close();
            rs_invoice = null;
         }
      }
      finally {
         if(rs_invoice != null)
            rs_invoice.close();
         if(rs_order != null)
            rs_order.close();
         if(pstmt != null)
            pstmt.close();
         if(stmt != null)
            stmt.close();
         if(conn != null)
            conn.close();
      }
      book.write(outputPath+"Invoice.xls");      
   }
   
   /* Part of createInvoice() */
   private static final void createInvoiceStyles(Workbook book)
   {
      Palette palette = book.getPalette();
      Color lightBlue = new Color(173, 216, 230);
      Color darkBlue = new Color(0, 0, 139);
      palette.setColor(55, darkBlue);
      palette.setColor(54, lightBlue);
      
      Style style;
      Font font;
      
      style = book.createStyle();
      font = style.getFont();
      font.setSize(12);
      font.setBold(true);
      font.setColor(Color.WHITE);
            
      style.setPatternStyle(PatternType.SOLID);
      style.setColor(lightBlue);
      style.setHAlignment(HorizontalAlignmentType.CENTRED);
      style.setName("Font12Center");
      
      style = (Style)style.clone();
      style.setHAlignment(HorizontalAlignmentType.LEFT);
      style.setName("Font12Left");
      
      style = (Style)style.clone();
      style.setHAlignment(HorizontalAlignmentType.RIGHT);
      style.setName("Font12Right");
      
      style = book.createStyle();
      style.setNumber(7);
      style.setName("Number7");
      
      style = book.createStyle();
      style.setNumber(9);
      style.setName("Number9");
      
      style = book.createStyle();
      style.setHAlignment(HorizontalAlignmentType.CENTRED);
      style.setName("Center");
      
      style = book.createStyle();
      font = style.getFont();
      font.setSize(16);
      font.setBold(true);
      font.setColor(darkBlue);
      style.setName("Darkblue");
      
      style = book.createStyle();
      font = style.getFont();
      font.setSize(12);
      font.setBold(true);
      font.setColor(darkBlue);
      style.setName("Darkblue12");
      
      style = book.createStyle();
      font = style.getFont();
      font.setItalic(true);
      font.setColor(darkBlue);
      style.setName("DarkblueItalic");
      
      style = book.createStyle();
      style.setBorderColor(BorderType.BOTTOM, Color.BLACK);
      style.setBorderLine(BorderType.BOTTOM, BorderLineType.MEDIUM);
      style.setName("BlackMedium");      
   }
   
   /* Part of createInvoice() */
   private static final void setColumnWidth(Worksheet sheet)
   {
      sheet.setColumnWidth(0, 12);
      sheet.setColumnWidth(1, 16);
      sheet.setColumnWidth(2, 16);
      sheet.setColumnWidth(3, 16);
      sheet.setColumnWidth(4, 16);
      sheet.setColumnWidth(5, 16);
      sheet.setColumnWidth(6, 18);
   }
   
   /* Part of createInvoice() */
   private static final void createInvoiceHeader(Workbook book, Worksheet sheet, ResultSet rs_invoice,
         int rowIndex) throws SQLException
   {
      Row row;
      Cell cell;
      Style style;
      String string;
      
      sheet.getRow(rowIndex).setHeight(24);
      
      cell = sheet.getCell(rowIndex, 5);
      cell.setValue("INVOICE");
      cell.setStyle(book.getNamedStyle("Darkblue"));
      
      style = book.getNamedStyle("BlackMedium");
      for(int i = 0; i <= 255; ++i)
         sheet.getCell(rowIndex + 2, i).setStyle(style);
      
      style = book.getNamedStyle("DarkblueItalic");
      cell = sheet.getCell(rowIndex + 3, 0);
      cell.setValue("One Portals Way, Twin Points WA 98156");
      cell.setStyle(style);
      cell = sheet.getCell(rowIndex + 4, 0);
      cell.setValue("Phone:1-206-555-1417 Fax:1-206");
      cell.setStyle(style);
      
      style = book.getNamedStyle("Darkblue12");
      cell = sheet.getCell(rowIndex + 3, 5);
      cell.setValue("Date:");
      cell.setStyle(style);
      
      Calendar calendar = new GregorianCalendar();
      StringBuilder timeDisplay = new StringBuilder();
      timeDisplay.append(calendar.get(Calendar.DAY_OF_MONTH)).append('/');
      timeDisplay.append(calendar.get(Calendar.MONTH)).append('/');
      timeDisplay.append(calendar.get(Calendar.YEAR));
      
      sheet.getCell(rowIndex + 3, 6).setValue(timeDisplay.toString());
      
      sheet.getRow(rowIndex + 6).setHeight(16);
      cell = sheet.getCell(rowIndex + 6, 0);
      cell.setValue("Ship To:");
      cell.setStyle(style);
      
      cell = sheet.getCell(rowIndex + 6, 4);
      cell.setValue("Bill To:");
      cell.setStyle(style);
      
      String tmp_str = rs_invoice.getString(1);
      sheet.getCell(rowIndex + 6, 1).setValue(tmp_str);
      sheet.getCell(rowIndex + 6, 5).setValue(tmp_str);
      
      tmp_str = rs_invoice.getString(2);
      sheet.getCell(rowIndex + 7, 1).setValue(tmp_str);
      sheet.getCell(rowIndex + 7, 5).setValue(tmp_str);
      
      tmp_str = "";
      tmp_str += rs_invoice.getString(3);
      tmp_str += rs_invoice.getString(4);
      tmp_str += rs_invoice.getString(5);
      tmp_str.trim();
      sheet.getCell(rowIndex + 8, 1).setValue(tmp_str);
      sheet.getCell(rowIndex + 8, 5).setValue(tmp_str);
      
      tmp_str = rs_invoice.getString(6);
      sheet.getCell(rowIndex + 9, 1).setValue(tmp_str);
      sheet.getCell(rowIndex + 9, 5).setValue(tmp_str);      
   }
   
   /* Part of createInvoice() */
   private static final void createOrder(Workbook book, Worksheet sheet, int rowIndex, 
         int orderID, ResultSet rs_invoice) throws SQLException
   {
      Style style;
      String str;
      Row row;
      
      sheet.getRow(rowIndex).setHeight(14);
      sheet.getRow(rowIndex + 3).setHeight(14);
      
      style = book.getNamedStyle("Font12Center");
      for(int i = 0; i < 7; ++i) {
         sheet.getCell(rowIndex, i).setStyle(style);
         sheet.getCell(rowIndex + 3, i).setStyle(style);
      }
      
      sheet.getCell(rowIndex, 0).setValue("Order ID:");
      sheet.getCell(rowIndex + 1, 0).setValue(orderID);
      
      sheet.getCell(rowIndex, 1).setValue("Customer ID:");
      sheet.getCell(rowIndex + 1, 1).setValue(rs_invoice.getString("CustomerID"));
      
      sheet.getCell(rowIndex, 2).setValue("Salesperson:");
      sheet.getCell(rowIndex + 1, 2).setValue(rs_invoice.getString("Salesperson"));      
      
      sheet.getCell(rowIndex, 3).setValue("Order Date:");
      sheet.getCell(rowIndex + 1, 3).setValue(rs_invoice.getString("OrderDate"));
      
      sheet.getCell(rowIndex, 4).setValue("Required Date:");
      sheet.getCell(rowIndex + 1, 4).setValue(rs_invoice.getString("RequiredDate"));
      
      sheet.getCell(rowIndex, 5).setValue("Shipped Date:");
      str = rs_invoice.getString("ShippedDate");
      if(str != null)
         sheet.getCell(rowIndex + 1, 5).setValue(str);
      
      sheet.getCell(rowIndex, 6).setValue("Ship Via");
      sheet.getCell(rowIndex + 1, 6).setValue(rs_invoice.getString(19));
      
      row = sheet.getRow(rowIndex + 3);
      row.getCell(0).setValue("Product ID:");
      row.getCell(1).setValue("Product");
      row.getCell(2).setValue("Name:");
      row.getCell(3).setValue("Quantity:");
      row.getCell(4).setValue("Unit Price:");
      row.getCell(5).setValue("Discount:");
      row.getCell(6).setValue("Extended Price:");
      
      row.getCell(1).setStyle(book.getNamedStyle("Font12Right"));
      row.getCell(2).setStyle(book.getNamedStyle("Font12Left"));
   }

   /**
    * Part of createInvoice().
    * @return how many rows written.
    */
   private static final int createOrderDetail(Workbook book, Worksheet sheet, int rowIndex, 
         ResultSet rs_invoice) throws SQLException
   {
      int rowIndexBackup = rowIndex;
      Style style_number7 = book.getNamedStyle("Number7");
      Style style_number9 = book.getNamedStyle("Number9");
      Style style_center = book.getNamedStyle("Center");
      
      rs_invoice.beforeFirst();
      while(rs_invoice.next()) {
         Row row = sheet.getRow(rowIndex);
         row.getCell(0).setValue(rs_invoice.getInt("ProductID"));
         row.getCell(0).setStyle(style_center);
         
         row.getCell(1).setValue(rs_invoice.getString("ProductName"));
         
         row.getCell(3).setValue(rs_invoice.getInt("Quantity"));
         
         row.getCell(4).setValue(rs_invoice.getDouble("UnitPrice"));
         row.getCell(4).setStyle(style_number7);
         
         row.getCell(5).setValue(rs_invoice.getDouble("Discount"));
         row.getCell(5).setStyle(style_number9);
         
         row.getCell(6).setValue(rs_invoice.getDouble("ExtendedPrice"));
         row.getCell(6).setStyle(style_number7);
         
         ++rowIndex;
      }
      
      return rowIndex - rowIndexBackup + 1;
   }
   //////////////////////////// END OF Create Invoice ///////////////////////////////////
   
   
   //////////////////////////// BEGIN OF Products by Category ///////////////////////////
   private static void createProductsByCategory(String outputPath) throws IOException, SQLException
   {
      Workbook book = new Workbook();
      book.read(TEMPLATE_FILE_PATH);
      Worksheet sheet = book.getSheet("Sheet7");
      sheet.setName("Products By Category");
      
      // remove useless sheets
      for(int i = book.getNumerOfSheets() - 1; i >= 0 ; --i) {
         if(book.getSheet(i) != sheet)
            book.removeSheet(i);
      }
      
      setProductsByCategoryStyles(book, sheet);
      int rowIndex = 4;
      int columnIndex = 0;
      int productsCount = 0;
      String lastCategory = "";
      String thisCategory, nextCagegory;
      
      Connection conn = null;
      Statement stmt = null;
      ResultSet rs = null;
      Cell cell;
      
      try {
         conn = DriverManager.getConnection(DB_URL);
         stmt = conn.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
         rs = stmt.executeQuery(
               "SELECT Categories.CategoryName, Products.ProductName, Products.QuantityPerUnit, " +
               "       Products.UnitsInStock, Products.Discontinued, Categories.CategoryID, " +
               "       Products.ProductID " +
               " FROM Categories INNER JOIN Products ON Categories.CategoryID = Products.CategoryID" +
               " WHERE (Products.Discontinued <> Yes) " +
               " ORDER BY Categories.CategoryName, Products.ProductName");
         
         while(rs.next()) {
            thisCategory = rs.getString("CategoryName");
            if(!thisCategory.equals(lastCategory)) {
               rowIndex = 4;
               if(!rs.isFirst())
                  columnIndex += 4;
               
               cell = sheet.getCell(rowIndex, columnIndex);
               cell.setValue("Category:");
               cell.setStyle(book.getNamedStyle("Category"));
               
               cell = sheet.getCell(rowIndex, columnIndex + 1);
               cell.setValue(thisCategory);
               cell.setStyle(book.getNamedStyle("CategoryName"));
               
               cell = sheet.getCell(rowIndex + 1, columnIndex);
               cell.setValue("Product Name");
               cell.setStyle(book.getNamedStyle("ProductName"));
               
               cell = sheet.getCell(rowIndex + 1, columnIndex + 1);
               cell.setValue("Units In Stock:");
               cell.setStyle(book.getNamedStyle("UnitsInStock"));
               
               lastCategory = thisCategory;
               rowIndex += 2;
            }
            sheet.getCell(rowIndex, columnIndex).setValue(rs.getString("ProductName"));
            sheet.getCell(rowIndex, columnIndex + 1).setValue(rs.getInt("UnitsInStock"));
            
            if(!rs.isLast()) {
               rs.next();
               nextCagegory = rs.getString("CategoryName");
               if(!thisCategory.equals(nextCagegory)) {
                  cell = sheet.getCell(rowIndex + 1, columnIndex);
                  cell.setValue("Number of Products:");
                  cell.setStyle(book.getNamedStyle("ProductsCount"));
                  
                  cell = sheet.getCell(rowIndex + 1, columnIndex + 1);
                  cell.setValue(productsCount + 1);
                  cell.setStyle(book.getNamedStyle("CountNumber"));
                  
                  ++rowIndex;
                  productsCount = 0;
               } else 
                  ++productsCount;
               rs.previous();
            } else {
               cell = sheet.getCell(rowIndex + 1, columnIndex);
               cell.setValue("Number of Products:");
               cell.setStyle(book.getNamedStyle("ProductsCount"));

               cell = sheet.getCell(rowIndex + 1, columnIndex + 1);
               cell.setValue(productsCount + 1);
               cell.setStyle(book.getNamedStyle("CountNumber"));
            }
            ++rowIndex;
         }
      }
      finally {
         if(rs != null)
            rs.close();
         if(stmt != null)
            stmt.close();
         if(conn != null)
            conn.close();
      }
      book.write(outputPath+"ProductsByCategory.xls");      
   }
   
   /* Part of createProductsByCategory */
   private static final void setProductsByCategoryStyles(Workbook book, Worksheet sheet)
   {
      sheet.getRow(4).setHeight(20.25f);
      sheet.getRow(5).setHeight(18.75f);
      
      Style style;
      Font font;
      
      style = book.createStyle();
      font = style.getFont();
      font.setItalic(true);
      font.setBold(true);
      font.setSize(16);
      style.setHAlignment(HorizontalAlignmentType.RIGHT);
      style.setName("Category");
      
      style = book.createStyle();
      font = style.getFont();
      font.setSize(16);
      font.setBold(true);
      style.setHAlignment(HorizontalAlignmentType.LEFT);
      style.setName("CategoryName");
      
      style = book.createStyle();
      font = style.getFont();
      font.setSize(14);
      font.setBold(true);
      font.setItalic(true);
      style.setHAlignment(HorizontalAlignmentType.LEFT);
      style.setBorderLine(BorderType.TOP, BorderLineType.MEDIUM);
      style.setBorderLine(BorderType.BOTTOM, BorderLineType.MEDIUM);
      style.setName("ProductName");
      
      style = book.createStyle();
      font = style.getFont();
      font.setSize(14);
      font.setBold(true);
      font.setItalic(true);
      style.setHAlignment(HorizontalAlignmentType.RIGHT);
      style.setBorderLine(BorderType.TOP, BorderLineType.MEDIUM);
      style.setBorderLine(BorderType.BOTTOM, BorderLineType.MEDIUM);
      style.setName("UnitsInStock");
      
      style = book.createStyle();
      font = style.getFont();
      font.setBold(true);
      font.setItalic(true);
      style.setBorderLine(BorderType.TOP, BorderLineType.THIN);
      style.setName("ProductsCount");
      
      style = book.createStyle();
      style.setHAlignment(HorizontalAlignmentType.LEFT);
      style.setBorderLine(BorderType.TOP, BorderLineType.THIN);
      style.setName("CountNumber");
   }
   //////////////////////////// END OF Products by Category /////////////////////////////
   
   
   /////////////////////////// BEGIN OF Sales by Category Subreport /////////////////////
   private static void createSalesByCategorySubreport(String outputPath) throws IOException, SQLException
   {
      Workbook book = new Workbook();
      book.read(TEMPLATE_FILE_PATH);
      Worksheet sheet = book.getSheet("Sheet9");
      sheet.setName("Sales By Category Subreport");
      
      // remove unuseful sheets
      for(int i = book.getNumerOfSheets() - 1; i >= 0 ; --i) {
         if(book.getSheet(i) != sheet)
            book.removeSheet(i);
      }
      
      Connection conn = null;
      Statement stmt = null;
      ResultSet rs = null;
      
      try {
         conn = DriverManager.getConnection(DB_URL);
         stmt = conn.createStatement();
         rs = stmt.executeQuery(
               "SELECT DISTINCTROW Products.ProductName, " +
               "       Sum([Order Details Extended].ExtendedPrice) AS ProductSales " + 
               " FROM Categories INNER JOIN (Products INNER JOIN " +
               "     (Orders INNER JOIN [Order Details Extended] ON " +
               "     Orders.OrderID = [Order Details Extended].OrderID) " +
               "     ON Products.ProductID = [Order Details Extended].ProductID)" +
               "     ON Categories.CategoryID = Products.CategoryID " +
               " WHERE (((Orders.OrderDate) Between #1/1/1995# And #12/31/1995#)) " +
               " GROUP BY Categories.CategoryID, Categories.CategoryName, Products.ProductName " +
               " ORDER BY Products.ProductName");
         
         sheet.importResultSet(rs, 0, 0, false);
      }
      finally {
         if(rs != null)
            rs.close();
         if(stmt != null)
            stmt.close();
         if(conn != null)
            conn.close();         
      }
      book.write(outputPath+"SalesByCategorySubreport.xls");
   }   
   /////////////////////////// END OF Sales by Category Subreport ///////////////////////
   
   
   /////////////////////////// BEGIN OF Sales by Year ///////////////////////////////////
   private static void createSalesByYear(String outputPath) throws IOException, SQLException
   {
      Workbook book = new Workbook();
      book.read(TEMPLATE_FILE_PATH);
      Worksheet sheet = book.getSheet("Sheet10");
      sheet.setName("Sales By Year");
      
      // remove unuseful sheets
      for(int i = book.getNumerOfSheets() - 1; i >= 0 ; --i) {
         if(book.getSheet(i) != sheet)
            book.removeSheet(i);
      }
      
      Connection conn = null;
      Statement stmt = null;
      ResultSet rs = null;
      
      try {
         conn = DriverManager.getConnection(DB_URL);
         stmt = conn.createStatement();
         rs = stmt.executeQuery(
               "SELECT DISTINCTROW ShippedDate, " +
               "        Orders.OrderID, [Order Subtotals].Subtotal as Subtotal " +
               " FROM Orders INNER JOIN [Order Subtotals] " +
               "      ON Orders.OrderID = [Order Subtotals].OrderID " +
               " WHERE( (Orders.ShippedDate) Is Not Null)");
         
         
         int rowIndex = 6;
         int row_num = 1;
         while(rs.next()) {
            sheet.getCell(rowIndex, 1).setValue(row_num++);
            String str = rs.getString(1);
            str = str.substring(0, str.indexOf('\40'));
            sheet.getCell(rowIndex, 2).setValue(str);
            sheet.getCell(rowIndex, 3).setValue(rs.getInt(2));
            sheet.getCell(rowIndex, 4).setValue(rs.getDouble(3));
            ++rowIndex;
         }
      }
      finally {
         if(rs != null)
            rs.close();
         if(stmt != null)
            stmt.close();
         if(conn != null)
            conn.close();         
      }
      book.write(outputPath+"SalesByYear.xls");
   }   
   /////////////////////////// END OF Sales By Year //////////////////////////////////////
   
   
   /////////////////////////// BEGIN OF Sales by Year ///////////////////////////////////
   private static void createSalesByYearSubreport(String outputPath) throws IOException, SQLException
   {
      Workbook book = new Workbook();
      book.read(TEMPLATE_FILE_PATH);
      Worksheet sheet = book.getSheet("Sheet11");
      sheet.setName("Sales By Year Subreport");
      
      // remove unuseful sheets
      for(int i = book.getNumerOfSheets() - 1; i >= 0 ; --i) {
         if(book.getSheet(i) != sheet)
            book.removeSheet(i);
      }
      
      Connection conn = null;
      Statement stmt = null;
      ResultSet rs = null;
      Cell cell;
      
      try {
         conn = DriverManager.getConnection(DB_URL);
         stmt = conn.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
         rs = stmt.executeQuery(
               "SELECT DISTINCTROW COUNT(*) AS Orders, " + 
               "  SUM([Order Subtotals].Subtotal) AS Sales, " +
               "  FORMAT(ORDERS.SHIPPEDDATE, 'yyyy/Q') AS Quarter " +
               " FROM Orders INNER JOIN [Order Subtotals] " +
               "    ON Orders.OrderID = [Order Subtotals].OrderID " +
               " WHERE (orders.shippeddate IS NOT NULL) " +
               " GROUP BY FORMAT(ORDERS.SHIPPEDDATE ,  'yyyy/Q')");
         
         int rowIndex = 0;
         int totalOrders = 0;
         double totalSales = 0;
         String thisYear = "";
         
         Style style;
         style = book.createStyle();
         style.getFont().setBold(true);
         style.setName("Bold");
         
         int loopCount = 0;
         while(rs.next()) {
            String quarter = rs.getString("Quarter");
            int orders = rs.getInt("Orders");
            double sales = rs.getDouble("Sales");
            
            if(loopCount == 0) {
               thisYear = quarter.substring(0, 4);
               createSalesByYearSubreportHeader(book, sheet, 0, thisYear);
               createSalesByYearSubreportData(sheet, 2, 0, quarter, orders, sales);
               totalOrders += orders;
               totalSales += sales;
               rowIndex = 3;
               
            } else {
               if(thisYear.equals(quarter.substring(0, 4))) {
                  createSalesByYearSubreportData(sheet, rowIndex, loopCount, quarter, orders, sales);
                  totalOrders += orders;
                  totalSales += sales;
                  ++rowIndex;
                  if(rs.isLast()) {
                     createSalesByYearSubreportFooter(book, sheet, rowIndex, totalOrders, totalSales);
                  }
               } else {
                  createSalesByYearSubreportFooter(book, sheet, rowIndex, totalOrders, totalSales);
                  totalOrders = 0;
                  totalSales = 0;
                  ++rowIndex;
                  thisYear = quarter.substring(0, 4);
                  if(!rs.isLast()) {
                     createSalesByYearSubreportHeader(book, sheet, rowIndex, thisYear);
                     rowIndex += 2;
                     createSalesByYearSubreportData(sheet, rowIndex, loopCount, 
                           quarter, orders, sales);
                     totalOrders += orders;
                     totalSales += sales;
                     ++rowIndex;                     
                  }
               }
            }
            ++loopCount;
         }
      }
      catch(SQLException ignored) {
         // Something went wrong. Well, ignore the exception and write out what we've got.
      }
      finally {
         if(rs != null)
            rs.close();
         if(stmt != null)
            stmt.close();
         if(conn != null)
            conn.close();         
      }
      book.write(outputPath+"SalesByYearSubreport.xls");
   }
   
   /* Part of createSalesByYearSubreport(). */
   private static final void createSalesByYearSubreportFooter(Workbook book, Worksheet sheet,
         int rowIndex, int totalOrders, double totalSales)
   {
      Style style = book.getNamedStyle("Bold");
      sheet.getCell(rowIndex, 1).setValue("Totals:");
      sheet.getCell(rowIndex, 1).setStyle(style);
      sheet.getCell(rowIndex, 2).setValue(totalOrders);
      sheet.getCell(rowIndex, 3).setValue(totalSales);
   }
   
   /* Part of createSalesByYearSubreport(). */
   private static final void createSalesByYearSubreportData(Worksheet sheet, int rowIndex, int index,
         String quarter, int orders, double sales) throws SQLException
   {
      sheet.getCell(rowIndex, 1).setValue(Integer.parseInt(quarter.substring(5)));
      sheet.getCell(rowIndex, 2).setValue(orders);
      sheet.getCell(rowIndex, 3).setValue(sales);
   }
   
   /* Part of createSalesByYearSubreport(). */
   private static final void createSalesByYearSubreportHeader(Workbook book, Worksheet sheet,
         int rowIndex, String year)
   {
      Style style = book.getNamedStyle("Bold");
      Cell cell;
      
      cell = sheet.getCell(rowIndex, 0);
      cell.setValue(year + " Summary");
      cell.setStyle(style);
      
      cell = sheet.getCell(rowIndex + 1, 1);
      cell.setValue("Quarter:");
      cell.setStyle(style);
      
      cell = sheet.getCell(rowIndex + 1, 2);
      cell.setValue("Orders Shipped:");
      cell.setStyle(style);

      cell = sheet.getCell(rowIndex + 1, 3);
      cell.setValue("Sales:");
      cell.setStyle(style);
   }
   /////////////////////////// END OF Sales By Year //////////////////////////////////////
   
   
   /////////////////////////// BEGIN OF Create Sales Totals //////////////////////////////
   private static void createSalesTotals(String outputPath) throws IOException, SQLException
   {
      Workbook book = new Workbook();
      book.read(TEMPLATE_FILE_PATH);
      Worksheet sheet = book.getSheet("Sheet12");
      sheet.setName("Sales Totals");
      
      // remove unuseful sheets
      for(int i = book.getNumerOfSheets() - 1; i >= 0 ; --i) {
         if(book.getSheet(i) != sheet)
            book.removeSheet(i);
      }
      
      Connection conn = null;
      Statement stmt = null;
      ResultSet rs = null;
      
      try {
         conn = DriverManager.getConnection(DB_URL);
         stmt = conn.createStatement();
         rs = stmt.executeQuery(
               "SELECT [Order Subtotals].Subtotal, [Order Subtotals].OrderID, " + 
               "    Customers.CompanyName, Customers.CustomerID " +
               " FROM Customers INNER JOIN ([Order Subtotals] INNER JOIN Orders " + 
               "    ON [Order Subtotals].OrderID = Orders.OrderID) " + 
               "    ON Customers.CustomerID = Orders.CustomerID " + 
               " WHERE (Orders.ShippedDate BETWEEN #1/1/1995# AND #12/31/1995#) " +
               "  AND ([Order Subtotals].Subtotal > 2500) " +  
               " ORDER BY [Order Subtotals].Subtotal DESC");
         
         int row_num = sheet.importResultSet(rs, 3, 1, -1, 3, false);
         
         double total = 0;
         for(int i = 3; i < 3 + row_num; ++i) {
            total += ((Double)sheet.getCell(i, 1).getValue()).doubleValue();
            sheet.getCell(i, 5).setValue(i - 2);
         }
         sheet.getCell(3 + row_num, 1).setValue(total);
         sheet.getCell(3 + row_num, 0).setValue("Total:");
         
         Style style = book.createStyle();
         style.getFont().setBold(true);
         sheet.getCell(3 + row_num, 0).setStyle(style);
      }
      finally {
         if(rs != null)
            rs.close();
         if(stmt != null)
            stmt.close();
         if(conn != null)
            conn.close();         
      }
      book.write(outputPath+"SalesTotals.xls");
   }      
   /////////////////////////// END OF Create Sales Totals ////////////////////////////////
   
   
   /////////////////////////// BEGIN OF Summary By Quarter ///////////////////////////////
   private static void CreateSummaryByQuarter(String outputPath) throws IOException, SQLException
   {
      Workbook book = new Workbook();
      book.read(TEMPLATE_FILE_PATH);
      Worksheet sheet = book.getSheet("Sheet13");
      sheet.setName("Summary By Quarter");
      
      // remove unuseful sheets
      for(int i = book.getNumerOfSheets() - 1; i >= 0 ; --i) {
         if(book.getSheet(i) != sheet)
            book.removeSheet(i);
      }
      
      Connection conn = null;
      Statement stmt = null;
      ResultSet rs = null;
      
      try {
         conn = DriverManager.getConnection(DB_URL);
         stmt = conn.createStatement();
         rs = stmt.executeQuery(
               "SELECT COUNT(Orders.OrderID) AS Orders, SUM([Order Subtotals].Subtotal) AS Sales, " +
               "       FORMAT(Orders.ShippedDate, 'yyyy/Q') AS Quarter " +
               " FROM Orders INNER JOIN [Order Subtotals] " +
               "     ON Orders.OrderID = [Order Subtotals].OrderID " + 
               " WHERE (Orders.ShippedDate IS NOT NULL) " +
               " GROUP BY FORMAT(Orders.ShippedDate, 'yyyy/Q')");
         
         int row_q1 = 5, row_q2 = 11, row_q3 = 17, row_q4 = 23;
         while(rs.next()) {
            int orders = rs.getInt(1);
            double sales = rs.getDouble(2);
            String quarter = rs.getString(3);
            
            int year = Integer.parseInt(quarter.substring(0, 4));
            switch(Integer.parseInt(quarter.substring(5))) {
            case 1:
               writeSummaryByQuarter(sheet, year, orders, sales, row_q1);
               ++row_q1;
               break;
            case 2:
               writeSummaryByQuarter(sheet, year, orders, sales, row_q2);
               ++row_q2;
               break;
            case 3:
               writeSummaryByQuarter(sheet, year, orders, sales, row_q3);
               ++row_q3;
               break;
            case 4:
               writeSummaryByQuarter(sheet, year, orders, sales, row_q4);
               ++row_q4;
               break;
            default:
               assert false;
            }
         }
      }
      finally {
         if(rs != null)
            rs.close();
         if(stmt != null)
            stmt.close();
         if(conn != null)
            conn.close();         
      }
      book.write(outputPath+"SummaryByQuarter.xls");
   }
   
   /* Part of CreateSummaryByQuarter(). */
   private static final void writeSummaryByQuarter(Worksheet sheet, int year, int orders, 
         double sales, int rowIndex)
   {
      sheet.getCell(rowIndex, 0).setValue(year);
      sheet.getCell(rowIndex, 1).setValue(orders);
      sheet.getCell(rowIndex, 2).setValue(sales);
   }
   /////////////////////////// END OF Summary By Quarter /////////////////////////////////   
}

